size_marker = 1.5
size_axes_title = 9
size_axes_text = 7
size_legend = 8
size_tile_border = 0.15
size_panel_border = 1

LEGEND_WIDTH_MAX = 30

LoadToEnvironment <- function(RData, env = new.env()){
  load(RData, env)
  return(env) 
}

CL_pub_label_list = list(`c32[humanmelanoma]`="C32")

MEANFILT_RPPA = -999
VARFILT_RPPA = 0.1
MEANFILT_RNASEQ = 2
VARFILT_RNASEQ = 0.5
MEANFILT_MUT = -999
VARFILT_MUT = 0.1

tissue2print = function(tissues) {
  #ret = gsub("haematopoieticandlymphoidtissue", "blood", tissues)
  ret = gsub("haematopoieticandlymphoidtissue", "haematopoietic\nand lymphoid", tissues)
  ret = gsub("largeintestine", "large intestine", ret)
  #ret = gsub("centralnervoussystem", "nerve", ret)
  ret = gsub("centralnervoussystem", "central nervous\nsystem", ret)
  ret = gsub("upperaerodigestivetract", "updigtract", ret)
  ret = gsub("all", "pan-cancer", ret)
  return(ret)
}

quantiles_95 <- function(x) {
  r <- quantile(x, probs=c(0.05, 0.25, 0.5, 0.75, 0.95))
  names(r) <- c("ymin", "lower", "middle", "upper", "ymax")
  r
}

quantiles_90 <- function(x) {
  r <- quantile(x, probs=c(0.10, 0.25, 0.5, 0.75, 0.90))
  names(r) <- c("ymin", "lower", "middle", "upper", "ymax")
  r
}

firstup <- function(x) {
  substr(x, 1, 1) <- toupper(substr(x, 1, 1))
  x
}

firstlow <- function(x) {
  substr(x, 1, 1) <- tolower(substr(x, 1, 1))
  x
}



GT_DRUGS_LEVELS = c("MAPK","PI3K","Chemo","HDAC","CHK1","Other")
CB_PALETTE = c("#E69F00", "#56B4E9", "#009E73", "#0072B2", "#D55E00", "#CC79A7", "#F0E442") # "#999999"
DRUGTYPE_GT_COLOR = c("MAPK"="#D55E00","PI3K"="#56B4E9","Chemo"="#009E73", "HDAC"="#0072B2","CHK1"="#E69F00","Other"="#999999")
DRUGTYPE_CCLE_COLOR = c("RTK"="#E69F00", "PI3K/mTOR"="#56B4E9", "Cell cycle"="#0072B2", "Cell cycle + PI3K/mTOR"="#F0E442", "JAK/STAT"="#CC79A7")
DRUGTYPE_GT_COLOR_AKT = c("AKT"="#CC79A7", "MAPK"="#D55E00","PI3K"="#56B4E9","Chemo"="#009E73", "HDAC"="#0072B2","CHK1"="#E69F00","Other"="#999999")
MAX_DRUGTARGETS = length(CB_PALETTE)+1


TISSUES = c("kidney", "liver", "urinarytract", "stomach", "oesophagus", "endometrium", "upperaerodigestivetract", "largeintestine", "centralnervoussystem", "breast", "pancreas", "ovary", "skin", "haematopoieticandlymphoidtissue", "lung", "all")
TISSUES_W_SYNS = c("kidney", "liver", "urinarytract", "stomach", "oesophagus", "endometrium", "upperaerodigestivetract", "updigtract", "largeintestine", "centralnervoussystem", "nerve", "breast", "pancreas", "ovary", "skin", "haematopoieticandlymphoidtissue", "blood", "lung", "all")
TISSUES_PRINT = tissue2print(TISSUES)
TISSUES_CCLE_INCLUDE = list(ctrp=c("lung", "largeintestine", "skin", "centralnervoussystem", "nerve", "ovary", "breast", "pancreas", "all"),
                       ccle=c("lung", "haematopoieticandlymphoidtissue", "blood", "skin", "all"))

TISSUE_CCLE_ALL = c("lung", "haematopoieticandlymphoidtissue", "skin", "centralnervoussystem", "breast", "ovary", "pancreas", "largeintestine", "all" )
TISSUE_CCLE = c("lung", "haematopoieticandlymphoidtissue", "skin", "centralnervoussystem", "breast", "ovary", "pancreas", "largeintestine" )
TISSUE_CCLE_CCLE_ALL = c("lung", "haematopoieticandlymphoidtissue", "skin", "centralnervoussystem", "breast", "ovary", "pancreas", "largeintestine", "all" )
TISSUE_CCLE_CTRP_ALL = c("lung", "skin", "centralnervoussystem", "breast", "ovary", "pancreas", "largeintestine", "all" )
TISSUE_GT = c("skin", "endometrium")
SEN_CCLE = c( "ccle", "ctrp" )
SEN_GT = c( "gt" )
SPEED1_ALL = c( "tnfa", "jakstat", "pi3konly", "wnt", "vegf", "mapkonly", "tgfb", "il1", "mapkpi3k", "tlr", "h2o2", "all" )
SPEED1_ALL = paste('speed1', SPEED1_ALL, sep="")
VERSIONS_RNASEQ_GT = c(SPEED1_ALL, c("rppaskin", "cosmic", "random30", "random50", "random100", "random500","unfilt", "-"))
VERSIONS_RNASEQ_CCLE = c(SPEED1_ALL, c("rppaccle", "cosmic", "random30", "random50", "random100", "random500","unfilt", "-"))
DATATYPE = c( "mut", "rppa", "rnaseq" )
DATATYPE_NOMUT = c( "rppa", "rnaseq" )


rppa_pub_label_list = list(`Akt.S473`=expression(Akt^{S473}), 
                           `NDRG1.S330`=expression(NDRG1^{S330}),
                           `p16.INK.4A`=expression(p16~INK~"4A"),
                           `Cofilin.S3`=expression(Cofilin^{S3}),
                           `Cyclin.D1`=expression(Cyclin~D1),
                           `S6.Ribo.Protein.S235.236`=expression(RPS6^{S235+236}),
                           `PRAS40.T246`=expression(PRAS40^{T246}),
                           `erbB2.Oncoprotein`=expression(erbB2),
                           `GSK.3.a.b.S21.9`=expression(GSK3ab^{S21/9}),
                           `RSK3.T356.S360`=expression(RSK3^{T356+S360}),
                           `Bad.S112`=expression(Bad^{S112}),
                           `Akt.T308`=expression(Akt^{T308}), 
                           `FKHR.T24.FKHRL1.T32`=expression(FKHR^{T24}~FKHRL1^{T32}), 
                           `ATP.Citrate.Lyase.S455`=expression(ATP~Citrate~Lyase^{S455}), 
                           `LDHA.LDHC`=expression(LDHA/LDHC), 
                           `Src.Y527` = expression(Src^{Y527}),
                           `p90.RSK.S380` = expression(p90RSK^{S380}),
                           `eIF2a.S51` = expression(eIF2a^{S51}),
                           `PKC.pan.beta.II.S660` = expression(PKCb^{S660}),
                           `Erk.1.2.T202.Y204` = expression(Erk^{T202+Y204}),
                           `cPLA2.S505` = expression(cPLA2^{S505}),
                           `LIMK1.T508.LIMK2.T505` = expression(LIMK1^{T508}~LIMK2^{T505}),
                           `FAK.Y576.577` = expression(FAK^{Y576+577}),
                           `IkappaB.a.S32` = expression(IkappaBa^{S32}),
                           `Tuberin.TSC2.Y1571` = expression(Tuberin.TSC2^{Y1571}),
                           `Stat3.Y705` = expression(Stat3^{Y705}),
                           `CREB.S133` = expression(CREB^{S133}),
                           `ELK.1.S383` = expression(ELK1^{S383}),
                           `MEK.1.2.S217.221` = expression(MEK1.2^{S217+221}),
                           `b.Raf.S445` = expression(bRaf^{S445}),
                           `p70.S6.Kinase.T389.T412` = expression(p70S6K^{T389+412}),
                           `eIF4e.S209` = expression(eIF4e^{S209}),
                           `c.Raf.S338` = expression(cRaf^{S338}),
                           `eIF4G.S1108` = expression(eIF4G^{S1108}),
                           `Ephrin.B.Y324.329` = expression(EphrinB^{Y324+329}),
                           `mTOR.S2448` = expression(mTOR^{S2448}),
                           `c.Raf.S259` = expression(cRaf^{S259}),
                           `Src.Family.Y416` = expression(Src^{Y416}),
                           `MEK1.S298` = expression(MEK1^{S298}),
                           `MEK1.S298` = expression(MEK1^{S298}),
                           `Jak2.Y1007.1008` = expression(Jak2^{Y1007+1008}),
                           `Jak2.Y1007.1008` = expression(Jak2^{Y1007+1008}),
                           `SAPK.JNK.T183.Y185` = expression(SAPK.JNK^{T183+Y185})
)

rppa_pub_label = function(rppa_labels) {
  ret = c()
  for(prot in rppa_labels) {
    if(prot %in% names(rppa_pub_label_list)) {
      ret = c(ret, rppa_pub_label_list[[prot]])
    } else {
      ret = c(ret, prot)
    }
  }
  return(ret)
}

ordered_by_freq  = function(x) {
  return(ordered(x, levels=names(table(x))[order(table(x), decreasing = T)]))
}

drug_type_label = function(drug_targets) {
  
  # Sort drug types by frequency (if not GT)
  drug_tagets_table = sort(table(drug_targets), decreasing=TRUE)
  drug_targets_filt = c()
  nDrugTargets = 0
  for(i in unique(drug_tagets_table)) {
    if(nDrugTargets + length(which(drug_tagets_table==i))<=MAX_DRUGTARGETS) {
      drug_targets_filt = c(drug_targets_filt, names(which(drug_tagets_table==i)))
      nDrugTargets = nDrugTargets + length(which(drug_tagets_table==i))
    }
  }
  
  # sort most common drug types alphabetically
  drug_targets_new = drug_targets
  if(any(!(drug_targets_new %in% drug_targets_filt)))
    drug_targets_new[!(drug_targets_new %in% drug_targets_filt)] = "other"
  drug_targets_new = ordered(drug_targets_new, levels=c(sort(setdiff(drug_targets_filt,c("other","Other"))), "other","Other"))
  drug_targets = drug_targets_new
  
  # line break for long strings
  drug_targets = str_wrap_factor(drug_targets, LEGEND_WIDTH_MAX )
  
  drug_target_color=c()
  drug_match_gt = match(unique(drug_targets), names(DRUGTYPE_GT_COLOR))
  drug_target_color = DRUGTYPE_GT_COLOR[drug_match_gt[!is.na(drug_match_gt)]]
  drug_match_ccle = match(unique(drug_targets), names(DRUGTYPE_CCLE_COLOR))
  drug_target_color = c(drug_target_color,DRUGTYPE_CCLE_COLOR[drug_match_ccle[!is.na(drug_match_ccle)]])
  if("other" %in% drug_targets)
    drug_target_color=c("other"="#999999")
  i=1
  cb_pal_unused = setdiff(CB_PALETTE, drug_target_color)
  unmatched_drugs = unique(drug_targets)[is.na(drug_match_gt) & is.na(drug_match_ccle) ]
  for(dt in setdiff(unmatched_drugs,c("Other","other"))) {
    drug_target_color = c(drug_target_color, cb_pal_unused[i])
    names(drug_target_color)[length(drug_target_color)] = dt
    i = i+1
  }
  
  return(list(drug_targets=drug_targets, drug_target_color=drug_target_color))
  
}


read_analysis_data = function(df_input, return_outs=F) {
  out_list = list()
  analysis_data = tibble()
  for(r in 1:nrow(df_input)) {
    datasource =  df_input[r,"datasource"]
    datatype = df_input[r,"datatype"]
    version = df_input[r,"version"]
    tissue = df_input[r,"tissue"]
    sen = df_input[r,"sen"]
    method = df_input[r,"method"]
    matched = df_input[r,"matched"]
    remove_dups = df_input[r,"remove_dups"]
    integration = df_input[r,"integration"]
    filename = paste("analysis/analysis_",datasource,"_", datatype,"_version",version,"_", 
                     tissue, "_sen", sen, "_", method, "_matched", 
                     matched, "_removeDups",remove_dups, "_integration",
                     integration, ".RData",sep="")
    if(!(file.exists(filename))) {
      print(paste("Missing:", filename))
    } else {
      
      loadenv = LoadToEnvironment(filename)
      out = loadenv$out
      if(!is.null(out)) {
        out_list[[length(out_list)+1]] = out
      } else {
        print(paste("No results in:", filename))
        next
      }
      n = length(out$cors)
      tissue_print = tissue2print(tissue)
      tmp = tibble(datasource=rep(datasource, n), 
                   datatype=rep(datatype, n), 
                   version=rep(version, n), 
                   tissue=rep(tissue_print, n),
                   sen=rep(sen,n),
                   method=rep(method, n), 
                   matched=rep(matched, n), 
                   remove_dups=rep(remove_dups, n), 
                   integration=rep(integration, n), 
                   drug = names(out$cors),
                   drug_target = out$drug_targets,
                   cor = out$cors,
                   varExpl = out$varsExpl,
                   pval_cor = out$pvals_cors,
                   pval_varExpl = out$pvals_varsExpl )
      
      analysis_data = rbind(analysis_data, tmp)
    }
  }
  if(return_outs) {
    return(out_list)
  } else {
    return(analysis_data)
  } 
}

filter_tissues = function(df) {
  
  TISSUES_CCLE_INCLUDE_FLAT = c()
  for(i in 1:length(TISSUES_CCLE_INCLUDE)) {
    TISSUES_CCLE_INCLUDE_FLAT = c(TISSUES_CCLE_INCLUDE_FLAT, paste(names(TISSUES_CCLE_INCLUDE)[i], TISSUES_CCLE_INCLUDE[[i]], sep="-"))
  }

  df = df %>% 
    filter(paste(sen, tissue,sep='-') %in% TISSUES_CCLE_INCLUDE_FLAT)
  return(df)
}

datatype2print = function(dt) {
  ret = gsub("rppa", "Protein", dt)
  ret = gsub("rnaseq", "mRNA", ret)
  ret = gsub("mut", "Mutation", ret)
  return(ret)
}

sen2print = function(sen) {
  ret = gsub("ccle", "CCLE", sen)
  ret = gsub("ctrp", "CTRP", ret)
  return(ret)
}